# Bigloo uses the terminology "release" for what Fedora calls version,
# and "version" for a sub-version revision.
# patch_suffix is defined to be empty when patch_ver is not defined,
# so that when updating, the Source and %%setup lines do not have to be
# changed, only the Version and patch_ver
#global patch_ver 5
#global patch_suffix %%{?patch_ver:-%%{patch_ver}}

# For Emacs subpackages
%global pkg     %{name}
%global pkgname Bigloo

Name:           bigloo
Version:        3.7a
Release:        3%{?patch_ver:.%{patch_ver}}%{?dist}
Summary:        A compiler for the Scheme programming language

Group:          Development/Languages
License:        GPLv2+
URL:            http://www-sop.inria.fr/mimosa/fp/Bigloo
Source0:        ftp://ftp-sop.inria.fr/indes/fp/Bigloo/bigloo%{version}%{?patch_suffix}.tar.gz
# Applied upstream: fix some bugs in the sendfile implementation.
Patch0:         bigloo-sendfile.patch
# Not yet sent upstream: fix various places where control falls off the end
# of a function that is declared to return a value.
Patch1:         bigloo-return.patch
# Not yet sent upstream: fix some bugs in the Emacs interface, and also
# modernizes the code somewhat.
Patch2:         bigloo-emacs.patch
# Applied upstream: fix a syntax error in a Makefile.
Patch3:         bigloo-makefile.patch

BuildRequires:  emacs
BuildRequires:  xemacs
BuildRequires:  xemacs-packages-extra
BuildRequires:  indent
BuildRequires:  java-devel >= 1:1.6.0
BuildRequires:  jpackage-utils
BuildRequires:  pkgconfig
BuildRequires:  zip
BuildRequires:  alsa-lib-devel
BuildRequires:  dbus-devel
BuildRequires:  flac-devel
BuildRequires:  gc-devel
BuildRequires:  gmp-devel
BuildRequires:  gstreamer-devel
BuildRequires:  gstreamer-plugins-base-devel
BuildRequires:  openssl-devel
BuildRequires:  sqlite-devel

Requires:       bigloo-libs%{?_isa} = %{version}-%{release}
Requires:       indent
Requires:       gmp-devel%{?_isa}
Requires:       gc-devel%{?_isa}
Requires:       gcc%{?_isa}
Requires(post):    info
Requires(postun):  info

ExcludeArch:    ppc64

%description
Bigloo is a Scheme implementation devoted to one goal: enabling a Scheme
based programming style where C(++) is usually required.  Bigloo
attempts to make Scheme practical by offering features usually presented
by traditional programming languages but not offered by Scheme and
functional programming.  Bigloo compiles Scheme modules.  It delivers
small and fast standalone binary executables.  Bigloo enables full
connections between Scheme and C programs.


%package libs
Summary:        Bigloo runtime libraries
Group:          System Environment/Libraries

%description libs
Runtime libraries for Bigloo compiled programs.


%package doc
Summary:        Bigloo documentation
Group:          Documentation
BuildArch:      noarch

%description doc
Documentation for the Bigloo compiler and integrated development
environment.


%package -n emacs-%{pkg}
Summary:        Compiled elisp files to run %{pkgname} under GNU Emacs
Group:          Development/Languages
BuildArch:      noarch
Requires:       emacs(bin) >= %{_emacs_version}

%description -n emacs-%{pkg}
This package contains the byte compiled elisp files to run %{pkgname}
with GNU Emacs.

%package -n emacs-%{pkg}-el
Summary:        Elisp source files for %{pkgname} under GNU Emacs
Group:          Development/Languages
BuildArch:      noarch
Requires:       emacs-%{pkg} = %{version}-%{release}

%description -n emacs-%{pkg}-el
This package contains the elisp source files for %{pkgname} under GNU Emacs. You
do not need to install this package to run %{pkgname}. Install the emacs-%{pkg}
package to use %{pkgname} with GNU Emacs.


%package -n xemacs-%{pkg}
Summary:        Compiled elisp files to run %{pkgname} under XEmacs
Group:          Development/Languages
BuildArch:      noarch
Requires:       xemacs(bin) >= %{_xemacs_version}
Requires:       xemacs-packages-extra

%description -n xemacs-%{pkg}
This package contains the byte compiled elisp files to run %{pkgname}
with XEmacs.

%package -n xemacs-%{pkg}-el
Summary:        Elisp source files for %{pkgname} under XEmacs
Group:          Development/Languages
BuildArch:      noarch
Requires:       xemacs-%{pkg} = %{version}-%{release}

%description -n xemacs-%{pkg}-el
This package contains the elisp source files for %{pkgname} under XEmacs. You do
not need to install this package to run %{pkgname}. Install the xemacs-%{pkg}
package to use %{pkgname} with XEmacs.


%prep
%setup -q -n %{name}%{version}%{?patch_suffix}
%patch0
%patch1
%patch2
%patch3

# encoding fixes
for f in README examples/Socket/socket.scm manuals/bigloo.info-2 \
    manuals/bigloo.info-3; do
  iconv -f ISO8859-1 -t UTF8 $f -o $f.utf8
  touch -r $f $f.utf8
  mv -f $f.utf8 $f
done

# libraries need an SONAME of the form .so.0, not .so
sed -i 's/sharedsuffix=so/sharedsuffix=so.0/g' configure
mkdir -p lib/%{version}
(cd lib/%{version}
# need these links, so that the linker finds the libraries
ln -sf libbigloo_u-%{version}.so.0 libbigloo_u-%{version}.so
ln -sf libbigloo_s-%{version}.so.0 libbigloo_s-%{version}.so)

# correct examples Makefiles for installation
find examples -name Makefile -print0 | xargs -0 \
     sed -i 's|include.*Makefile.config|include %{_libdir}/bigloo/%{version}/Makefile.config|g'

# fix a broken test
sed -i "s|-o recette\$(EXE_SUFFIX)|-o recette\$(EXE_SUFFIX) -lgstbase-0.10|" \
    api/gstreamer/recette/Makefile


%build
%define inplace `pwd`/inplace
%define bigloo_opt_flags "`echo $RPM_OPT_FLAGS | sed 's/-Wall/-w/g' | sed 's/FORTIFY_SOURCE=./FORTIFY_SOURCE=0/g'`"

CFLAGS="%bigloo_opt_flags"
export CFLAGS
sed -i -e "s/^jcflags=-O$/jcflags=-deprecation/" configure
./configure \
        --prefix=%{_prefix} \
        --bindir=%{_bindir} \
        --libdir=%{_libdir} \
        --mandir=%{_mandir}/man1 \
        --infodir=%{_infodir} \
        --docdir=%{_docdir} \
        --lispdir=%{_emacs_sitelispdir}/bigloo \
        --jvm=yes \
        --javac=javac \
        --bee=full \
        --customgc=no \
        --coflags="$CFLAGS" \
        --sharedbde=yes \
        --sharedcompiler=yes \
        --native-default-backend \
        --dotnet=no \
        --customgmp=no \
        --strip=no \
        --configureinfo=yes

# _smp_mflags breaks the build
env LD_LIBRARY_PATH=`pwd`/lib/%{version} \
    BIGLOOLIB=%{inplace}%{_libdir}/bigloo/%{version} \
    make
env LD_LIBRARY_PATH=`pwd`/lib/%{version} \
    BIGLOOLIB=%{inplace}%{_libdir}/bigloo/%{version} \
    make DESTDIR=%{inplace} install
env LD_LIBRARY_PATH=`pwd`/lib/%{version} \
    PATH=`pwd`/bin:$PATH \
    BIGLOOLIB=%{inplace}%{_libdir}/bigloo/%{version} \
    make compile-bee


%install
env LD_LIBRARY_PATH=`pwd`/lib/%{version} \
    make DESTDIR=$RPM_BUILD_ROOT INSTALL="cp -a" install
env LD_LIBRARY_PATH=`pwd`/lib/%{version} \
    make DESTDIR=$RPM_BUILD_ROOT EMACSDIR=%{_emacs_sitelispdir}/bigloo install-bee
make -C manuals DESTDIR=$RPM_BUILD_ROOT install-bee

# fix permissions
chmod 755 $RPM_BUILD_ROOT%{_bindir}/*

# move libraries to proper place
# and rename them to conform to SONAME
(
    cd $RPM_BUILD_ROOT%{_libdir}
    chmod 755 bigloo/%{version}/*.so.0
    rm -f *.so.0
    mv bigloo/%{version}/*.so.0 .
    for f in *.so.0
    do
        mv $f ${f%%.so.0}.so.0.0.0
        ln -sf ${f%%.so.0}.so.0.0.0 $f
        ln -sf $f ${f%%.so.0}.so
    done
    (cd bigloo/%{version}; ln -sf ../../*.so .)
)

sed -e 's|^BOOTBINDIR.*|BOOTBINDIR=%{_bindir}|g' \
    -e 's|^BOOTLIBDIR.*|BOOTLIBDIR=%{_libdir}/bigloo/%{version}|g' \
    -i $RPM_BUILD_ROOT%{_libdir}/bigloo/%{version}/Makefile.config

rm -fr $RPM_BUILD_ROOT%{_infodir}/dir
rm -fr $RPM_BUILD_ROOT%{_datadir}/doc

# emacs
mkdir -p $RPM_BUILD_ROOT%{_emacs_sitestartdir}
cat > $RPM_BUILD_ROOT%{_emacs_sitestartdir}/bigloo.el <<EOF
(require 'bmacs)
EOF
pushd $RPM_BUILD_ROOT%{_emacs_sitelispdir}/bigloo
rm -f bmacs-xemacs.el xemacs-etags.el
%{_emacs_bytecompile} bmacs.el bmacs-config.el bmacs-gnu-emacs.el
popd

# xemacs
pushd bmacs
make clean
rm -f bmacs-gnu-emacs.el bug/bug-gnu-emacs.el
sed -i -e '/bmacs-gnu-emacs\.el/D' -e 's/ bug-gnu-emacs//' Makefile
make EMACS=xemacs EMACSBRAND=xemacs EMACSDIR=$RPM_BUILD_ROOT%{_xemacs_sitelispdir}/bigloo
make EMACS=xemacs EMACSBRAND=xemacs EMACSDIR=$RPM_BUILD_ROOT%{_xemacs_sitelispdir}/bigloo install
cd $RPM_BUILD_ROOT%{_xemacs_sitelispdir}/bigloo
mkdir -p ../../etc/bigloo
mv etc/* ../../etc/bigloo
rmdir etc
%{_xemacs_bytecompile} bmacs.el bmacs-config.el bmacs-xemacs.el xemacs-etags.el
popd

mkdir -p $RPM_BUILD_ROOT%{_xemacs_sitestartdir}
cat > $RPM_BUILD_ROOT%{_xemacs_sitestartdir}/bigloo.el <<EOF
(require 'bmacs)
EOF


%post
/sbin/install-info %{_infodir}/bigloo.info %{_infodir}/dir 2>/dev/null || :
/sbin/install-info %{_infodir}/bdb.info %{_infodir}/dir 2>/dev/null || :


%postun
if [ $1 = 0 ]; then
  /sbin/install-info --delete %{_infodir}/bigloo.info %{_infodir}/dir 2>/dev/null || :
  /sbin/install-info --delete %{_infodir}/bdb.info %{_infodir}/dir 2>/dev/null || :
fi


%post libs -p /sbin/ldconfig


%postun libs -p /sbin/ldconfig


%check
make test


%files
%{_bindir}/*
%{_libdir}/bigloo
%{_libdir}/lib*.so
%{_infodir}/*
%{_mandir}/man*/*
%doc Makefile.config examples LICENSE COPYING
%doc README*


%files -n emacs-bigloo
%{_emacs_sitelispdir}/bigloo
%exclude %{_emacs_sitelispdir}/bigloo/*.el
%{_emacs_sitestartdir}/bigloo.el

%files -n emacs-bigloo-el
%{_emacs_sitelispdir}/bigloo/*.el


%files -n xemacs-bigloo
%{_xemacs_sitelispdir}/bigloo
%exclude %{_xemacs_sitelispdir}/bigloo/*.el
%{_xemacs_sitestartdir}/bigloo.el
%{_xemacs_sitepkgdir}/etc/bigloo

%files -n xemacs-bigloo-el
%{_xemacs_sitelispdir}/bigloo/*.el


%files libs
%{_libdir}/lib*.so.0*


%files doc
%doc manuals/*.html


%changelog
* Mon Jan  9 2012 Jerry James <loganjerry@gmail.com> - 3.7a-3
- Rebuild for GCC 4.7
- Drop unnecessary BRs

* Wed Nov 23 2011 Jerry James <loganjerry@gmail.com> - 3.7a-2
- Fix (X)Emacs installations (bz 754592)
- Refresh sendfile patch to match upstream's version

* Thu Oct 20 2011 Marcela Mašláňová <mmaslano@redhat.com> - 3.7a-1.2
- rebuild with new gmp without compat lib

* Mon Oct 10 2011 Peter Schiffer <pschiffe@redhat.com> - 3.7a-1.1
- rebuild with new gmp

* Mon Sep 26 2011 Jerry James <loganjerry@gmail.com> - 3.7a-1
- Update to 3.7a
- New source URL
- Drop unnecessary spec file elements (BuildRoot, defattr, etc.)
- Reenable XEmacs subpackages
- Fix more files with non-UTF-8 encodings
- Four patches to fix problems encountered with the new version
- Add alsa-lib-devel, dbus-devel, and flac-devel BRs for new APIs
- The obsoletes were needed for now EOL Fedora releases; remove them
- Add check script

* Mon Feb 14 2011 Gérard Milmeister <gemi@bluewin.ch> - 3.6a-1
- Update to 3.6a

* Mon Feb 07 2011 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 3.4a-2
- Rebuilt for https://fedoraproject.org/wiki/Fedora_15_Mass_Rebuild

* Fri Jul  2 2010 Michel Salim <salimma@fedoraproject.org> - 3.4a-1
- Update to 3.4a

* Fri Jul  2 2010 Michel Salim <salimma@fedoraproject.org> - 3.3a-2.5
- Now complies with Emacs packaging guidelines
- Disable broken XEmacs subpackages

* Sun May  9 2010 Michel Salim <salimma@fedoraproject.org> - 3.3a-1.5
- Update to 3.3a-5

* Fri Aug 21 2009 Tomas Mraz <tmraz@redhat.com> - 3.2b-3
- rebuilt with new openssl

* Sun Aug 16 2009 Gerard Milmeister <gemi@bluewin.ch> - 3.2b-2
- added requires gc-devel

* Sun Aug  9 2009 Gerard Milmeister <gemi@bluewin.ch> - 3.2b-1
- new release 3.2b

* Fri Jul 24 2009 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 3.1b-6
- Rebuilt for https://fedoraproject.org/wiki/Fedora_12_Mass_Rebuild

* Mon Feb 23 2009 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 3.1b-5
- Rebuilt for https://fedoraproject.org/wiki/Fedora_11_Mass_Rebuild

* Thu Jan 15 2009 Tomas Mraz <tmraz@redhat.com> - 3.1b-4
- rebuild with new openssl

* Tue Sep 30 2008 Gerard Milmeister <gemi@bluewin.ch> - 3.1b-3
- add BR openssl-devel
- add missing Req. gmp-devel

* Thu Sep 18 2008 Gerard Milmeister <gemi@bluewin.ch> - 3.1b-1
- new release 3.1b

* Sat May 31 2008 Gerard Milmeister <gemi@bluewin.ch> - 3.1a-1
- new release 3.1a

* Mon Apr 14 2008 Gerard Milmeister <gemi@bluewin.ch> - 3.0c-4.1
- new release 3.0c-4

* Mon Feb 18 2008 Fedora Release Engineering <rel-eng@fedoraproject.org> - 3.0b-2
- Autorebuild for GCC 4.3

* Tue Sep 11 2007 Gerard Milmeister <gemi@bluewin.ch> - 3.0b-1
- new release 3.0b

* Sat Jun  2 2007 Gerard Milmeister <gemi@bluewin.ch> - 3.0a-4
- exclude ppc64

* Fri Jun  1 2007 Gerard Milmeister <gemi@bluewin.ch> - 3.0a-3
- remove java ssl since it does not build with libgcj

* Fri Jun  1 2007 Gerard Milmeister <gemi@bluewin.ch> - 3.0a-1
- new version 3.0a

* Wed Mar 28 2007 Thomas Fitzsimmons <fitzsim@redhat.com> - 2.9a-3
- Patch method calls for Java 1.5.

* Tue Mar 27 2007 Thomas Fitzsimmons <fitzsim@redhat.com> - 2.9a-2
- Require java-1.5.0-gcj-devel for build.

* Thu Dec 28 2006 Gerard Milmeister <gemi@bluewin.ch> - 2.9a-1
- new version 2.9a

* Wed Sep 13 2006 Gerard Milmeister <gemi@bluewin.ch> - 2.8c-5
- final version 2.8c

* Tue Sep  5 2006 Gerard Milmeister <gemi@bluewin.ch> - 2.8c-4
- reenable hard links

* Fri Sep  1 2006 Gerard Milmeister <gemi@bluewin.ch> - 2.8c-2
- updated to 2.8c-beta01Sep06

* Mon Aug 28 2006 Gerard Milmeister <gemi@bluewin.ch> - 2.8b-3
- Rebuild for FE6

* Thu Jun 22 2006 Gerard Milmeister <gemi@bluewin.ch> - 2.8b-1
- new version 2.8b

* Thu Jun  1 2006 Gerard Milmeister <gemi@bluewin.ch> - 2.8a-2
- final 2.8a release

* Tue Mar 21 2006 Gerard Milmeister <gemi@bluewin.ch> - 2.8a-1.20060322
- new version 2.8a-20060322

* Mon Mar 13 2006 Gerard Milmeister <gemi@bluewin.ch> - 2.8a-1.20060313
- new version 2.8a-20060313

* Mon Nov 28 2005 Gerard Milmeister <gemi@bluewin.ch> - 2.7a-2
- disable bmem, causes link problems

* Sun Nov 27 2005 Gerard Milmeister <gemi@bluewin.ch> - 2.7a-1
- New Version 2.7a

* Tue Sep 27 2005 Gerard Milmeister <gemi@bluewin.ch> - 2.6f-3
- Removed --arch

* Fri Feb 18 2005 Gerard Milmeister <gemi@bluewin.ch> - 0:2.6f-1
- New Version 2.6f

* Sun Feb 13 2005 Gerard Milmeister <gemi@bluewin.ch> - 0:2.6e-1
- New Version 2.6e

* Tue Mar 16 2004 Gerard Milmeister <gemi@bluewin.ch> - 0:2.6-0.fdr.1.c
- New Version 2.6c
- Separated Emacs and XEmacs packages

* Tue Dec 16 2003 Gerard Milmeister <gemi@bluewin.ch> - 0:2.6-0.fdr.1.b
- New Version 2.6b

* Thu Nov 27 2003 Gerard Milmeister <gemi@bluewin.ch> - 0:2.6-0.fdr.1.a
- First Fedora release
